unit VaxDirectProxyList;

interface

uses
    System.SysUtils,
    System.Classes,
    Generics.Collections;

type TVaxDirectProxyDetails = class

  public
    m_sProxyName, m_sAuthLogin, m_sAuthPwd, m_sToPeerName : string;
    m_sDomainRealm, m_sProxyIP, m_sCodecList, m_sRoomName : string;
    m_lProxyPort, m_lToPeerType : Integer;

    m_bBlock, m_bAddToConf : Boolean;


end;

type  TVaxDirectProxyList = class

    var
        m_objVaxDirectProxy : TObjectDictionary<string, TVaxDirectProxyDetails>;

      private

        function GenKey(sValue : string) : string;
        function FindDirectProxy(sProxyName : string) : TVaxDirectProxyDetails;
        procedure DeleteAllDirectProxy();

      public

        constructor Create();
        Destructor Destroy();

        function IsDirectProxyExist(sProxyName : string) : boolean;

        procedure ResetOBJ();

        function AddDirectProxySIP(sProxyName, sAuthLogin, sAuthPwd, sDomainRealm, sProxyIP : string; lProxyPort : Integer; sCodecList : string; nToPeerType : Integer; sToPeerName : string; bBlock, bAddToConf : boolean; sRoomName : string) : boolean;
        procedure RemoveDirectProxySIP(sProxyName : string);

        function GetDirectProxyAll(var pProxyList : TStringList) : boolean;
        function GetDirectProxyDetails(sProxyName : string; var sAuthLogin, sAuthPwd, sDomainRealm, sProxyIP : string; var lProxyPort : Integer; var sCodecList : string; var nToPeerType : Integer; sToPeerName : string; var bBlock, bAddToConf : boolean; var sRoomName : string) : boolean;


end;

implementation

        constructor TVaxDirectProxyList.Create();
        begin
            m_objVaxDirectProxy := TObjectDictionary<string, TVaxDirectProxyDetails>.Create([doOwnsValues], 256);
        end;

        Destructor TVaxDirectProxyList.Destroy();
        begin
            DeleteAllDirectProxy();
            m_objVaxDirectProxy.Free;
        end;

        function TVaxDirectProxyList.GenKey(sValue : string) : string;
        begin
                Result := AnsiLowerCase(sValue);
        end;

        function TVaxDirectProxyList.FindDirectProxy(sProxyName : string) : TVaxDirectProxyDetails;
        var
            bResult : Boolean;
            objDirectProxy : TVaxDirectProxyDetails;
        begin
            bResult := m_objVaxDirectProxy.TryGetValue(AnsiLowerCase(sProxyName), objDirectProxy);
            if(bResult = False) then
                Exit(nil);

            Result := objDirectProxy;
        end;

        procedure TVaxDirectProxyList.DeleteAllDirectProxy();
        begin
            m_objVaxDirectProxy.Clear;
        end;

        function TVaxDirectProxyList.IsDirectProxyExist(sProxyName : string) : boolean;
        var
            objDirectProxy : TVaxDirectProxyDetails;
        begin
            objDirectProxy := FindDirectProxy(sProxyName);
            if(objDirectProxy = nil) then Exit(False);

            Exit(True);
        end;

        procedure TVaxDirectProxyList.ResetOBJ();
        begin
            DeleteAllDirectProxy();
        end;

        function TVaxDirectProxyList.AddDirectProxySIP(sProxyName, sAuthLogin, sAuthPwd, sDomainRealm, sProxyIP : string; lProxyPort : Integer; sCodecList : string; nToPeerType : Integer; sToPeerName : string; bBlock, bAddToConf : boolean; sRoomName : string) : boolean;
        var
            sKey : string;
            objDirectProxy : TVaxDirectProxyDetails;
        begin

            objDirectProxy := FindDirectProxy(sProxyName);
            if(IsDirectProxyExist(sProxyName) = True) then
                Exit(False);

            objDirectProxy := TVaxDirectProxyDetails.Create();

            objDirectProxy.m_sProxyName := sProxyName;
            objDirectProxy.m_sAuthLogin := sAuthLogin;
            objDirectProxy.m_sAuthPwd := sAuthPwd;

            objDirectProxy.m_sDomainRealm := sDomainRealm;
            objDirectProxy.m_sProxyIP := sProxyIP;
            objDirectProxy.m_lProxyPort := lProxyPort;

            objDirectProxy.m_sCodecList := sCodecList;

            objDirectProxy.m_lToPeerType := nToPeerType;
            objDirectProxy.m_sToPeerName := sToPeerName;
            objDirectProxy.m_bAddToConf := bAddToConf;

            objDirectProxy.m_bBlock := bBlock;
            objDirectProxy.m_sRoomName := sRoomName;

            sKey := GenKey(sProxyName);

            m_objVaxDirectProxy.Add(sKey, objDirectProxy);

            Result := True;

        end;

        procedure TVaxDirectProxyList.RemoveDirectProxySIP(sProxyName : string);
        var
            objDirectProxy : TVaxDirectProxyDetails;
        begin

            objDirectProxy := FindDirectProxy(sProxyName);
            if(objDirectProxy = nil) then Exit;

            m_objVaxDirectProxy.Remove(AnsiLowerCase(sProxyName));
            //FreeAndNil(objDirectProxy);
        end;

        function TVaxDirectProxyList.GetDirectProxyAll(var pProxyList : TStringList) : boolean;
        var
            objDirectProxy : TVaxDirectProxyDetails;
        begin

            for objDirectProxy in m_objVaxDirectProxy.Values do
              begin
                pProxyList.Add(objDirectProxy.m_sProxyName)
              end;

              Result := True;
        end;

        function TVaxDirectProxyList.GetDirectProxyDetails(sProxyName : string; var sAuthLogin, sAuthPwd, sDomainRealm, sProxyIP : string; var lProxyPort : Integer; var sCodecList : string; var nToPeerType : Integer; sToPeerName : string; var bBlock, bAddToConf : boolean; var sRoomName : string) : boolean;
        var
            objDirectProxy : TVaxDirectProxyDetails;
        begin

            objDirectProxy := FindDirectProxy(sProxyName);
            if(objDirectProxy = nil) then
                Exit(False);

            sAuthLogin := objDirectProxy.m_sAuthLogin;
            sAuthPwd := objDirectProxy.m_sAuthPwd;

            sDomainRealm := objDirectProxy.m_sDomainRealm;
            sProxyIP := objDirectProxy.m_sProxyIP;
            lProxyPort := objDirectProxy.m_lProxyPort;

            sCodecList := objDirectProxy.m_sCodecList;

            nToPeerType := objDirectProxy.m_lToPeerType;
            sToPeerName := objDirectProxy.m_sToPeerName;
            bAddToConf := objDirectProxy.m_bAddToConf;

            bBlock := objDirectProxy.m_bBlock;
            sRoomName := objDirectProxy.m_sRoomName;

            Result := True;

        end;


end.
