#include "StdAfx.h"
#include "CallPickUpList.h"
#include "CallPickUpMemberList.h"

CCallPickUpDetails::CCallPickUpDetails(LPCTSTR sPickUpGroupName)
{
	m_pMemberList = new CCallPickUpMemberList();
	m_sPickUpGroupName = sPickUpGroupName;
}

CCallPickUpDetails::~CCallPickUpDetails()
{
	delete m_pMemberList;
}

BOOL CCallPickUpDetails::AddCallPickUpGroupMember(LPCTSTR sMemberName)
{
	return m_pMemberList->AddMember(sMemberName);
}

void CCallPickUpDetails::RemoveCallPickUpGroupMember(LPCTSTR sMemberName)
{
	m_pMemberList->RemoveMember(sMemberName);
}

BOOL CCallPickUpDetails::GetPickUpMemberList(CStringList& sListMemberName)
{
	return m_pMemberList->GetMemberList(sListMemberName);
}

BOOL CCallPickUpDetails::IsGroupMember(LPCTSTR sMemberName)
{
	return m_pMemberList->IsGroupMember(sMemberName);
}

///////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////  CALLPICKUPLIST  ////////////////////////////////////////

CCallPickUpList::CCallPickUpList(void)
{
	m_pCallPickUpDetailsList = new CCallPickUpDetailsList();
	m_pCallPickUpDetailsList->InitHashTable(20);
}

CCallPickUpList::~CCallPickUpList(void)
{
	DeletePickUpGroupAll();
	delete m_pCallPickUpDetailsList;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////

CString CCallPickUpList::GenKey(LPCTSTR sPickUpGroupName)
{
	return CString(sPickUpGroupName).MakeLower();
}

void CCallPickUpList::DeletePickUpGroupAll()
{
	if(m_pCallPickUpDetailsList->GetCount() == 0)
		return;

	CCallPickUpDetails* pDetails = NULL;
	CString sKey;

	POSITION pPos = m_pCallPickUpDetailsList->GetStartPosition();

	while(pPos != NULL)
	{
		m_pCallPickUpDetailsList->GetNextAssoc(pPos, sKey, pDetails);
		
		if(pDetails != NULL)
			delete pDetails;
	}

	m_pCallPickUpDetailsList->RemoveAll();
}

CCallPickUpDetails* CCallPickUpList::FindCallPickUp(LPCTSTR sPickUpGroupName)
{
	CCallPickUpDetails* pDetails = NULL;

	CString sKey = GenKey(sPickUpGroupName);

	if(!m_pCallPickUpDetailsList->Lookup(sKey, pDetails))
		return NULL;

	return pDetails;
}

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

BOOL CCallPickUpList::GetAllPickUpGroupName(CStringList& sListPickUpName)
{
	POSITION posList =  m_pCallPickUpDetailsList->GetStartPosition();

	CString sPickUpGroupName;
	CCallPickUpDetails* pCallPickUpDetails; 

	while(posList != NULL)
	{
		m_pCallPickUpDetailsList->GetNextAssoc(posList, sPickUpGroupName, pCallPickUpDetails);
		sListPickUpName.AddTail(pCallPickUpDetails->m_sPickUpGroupName);
	}

	return TRUE;
}

BOOL CCallPickUpList::GetPickUpMemberName(LPCTSTR sPickUpGroupName, CStringList& sListMemberName)
{
	POSITION posList =  m_pCallPickUpDetailsList->GetStartPosition();
	
	CCallPickUpDetails* pCallPickUpDetails = NULL; 

	CString sKey = GenKey(sPickUpGroupName);

	m_pCallPickUpDetailsList->Lookup(sKey, pCallPickUpDetails);

	if(pCallPickUpDetails == NULL)
		return FALSE;

	return pCallPickUpDetails->GetPickUpMemberList(sListMemberName);
}

//////////////////////////////////////////////////////////////////////////////

BOOL CCallPickUpList::AddCallPickUpGroup(LPCTSTR sPickUpGroupName)
{
	if(CString(sPickUpGroupName).IsEmpty())
		return FALSE;
	
	if(FindCallPickUp(sPickUpGroupName) != NULL)
		return FALSE;

	CCallPickUpDetails* pDetails = new CCallPickUpDetails(sPickUpGroupName);
	
	CString sKey = GenKey(sPickUpGroupName);

	m_pCallPickUpDetailsList->SetAt(sKey, pDetails);

	return TRUE;
}

void CCallPickUpList::RemoveCallPickUpGroup(LPCTSTR sPickUpGroupName)
{
	CCallPickUpDetails* pDetails = NULL;

	CString sKey = GenKey(sPickUpGroupName);

	if(!m_pCallPickUpDetailsList->Lookup(sKey, pDetails))
		return;

	delete pDetails;

	m_pCallPickUpDetailsList->RemoveKey(sKey);

}

BOOL CCallPickUpList::AddCallPickUpGroupMember(LPCTSTR sPickUpGroupName, LPCTSTR sMemberName)
{
	CCallPickUpDetails* pDetails = NULL;
	CString sKey = GenKey(sPickUpGroupName);

	if(!m_pCallPickUpDetailsList->Lookup(sKey, pDetails))
		return FALSE;

	if(pDetails == NULL)
		return FALSE;
	return pDetails->AddCallPickUpGroupMember(sMemberName);
}

void CCallPickUpList::RemoveCallPickUpGroupMember(LPCTSTR sPickUpGroupName, LPCTSTR sMemberName)
{
	CCallPickUpDetails* pDetails = NULL;
	CString sKey = GenKey(sPickUpGroupName);

	if(!m_pCallPickUpDetailsList->Lookup(sKey, pDetails))
		return;

	if(pDetails == NULL)
		return;

	pDetails->RemoveCallPickUpGroupMember(sMemberName);
}

BOOL CCallPickUpList::IsGroupMember(LPCTSTR sPickUpGroupName, LPCTSTR sMemberName)
{
	CCallPickUpDetails* pDetails = NULL;
	CString sKey = GenKey(sPickUpGroupName);

	if(!m_pCallPickUpDetailsList->Lookup(sKey, pDetails))
		return 0;

	if(pDetails == NULL)
		return 0;

	return pDetails->IsGroupMember(sMemberName);
}

