﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Collections;

namespace ServerSIP
{
    public partial class CallPickupForm : Form
    {
        private cVaxServerSIP m_objVaxTeleSIP;

        public CallPickupForm(ref cVaxServerSIP objVaxServer)
        {
            InitializeComponent();
            m_objVaxTeleSIP = objVaxServer;
        }

        private void BtnAddCallPickupGroup_Click(object sender, EventArgs e)
        {
            if(comboPickupGroup.Text == "")
            {
                MessageBox.Show("Please enter pickup group name.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }

            if(comboPickupGroup.FindStringExact(comboPickupGroup.Text) != -1)
            {
                if(MessageBox.Show("Pickup Group already exists, do you want to continue?", "VaxTeleSIP", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == DialogResult.No)
                {
                    return;
                }
            }

            if(comboPickupGroup.Items.Contains(comboPickupGroup.Text))
                m_objVaxTeleSIP.RemoveCallPickUpGroup(comboPickupGroup.Text);

            if(!m_objVaxTeleSIP.AddCallPickUpGroup(comboPickupGroup.Text))
            {
                MessageBox.Show("Pickup Group couldn't be added.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            foreach (string sMemeberName in ListBoxMember.Items)
            {
                m_objVaxTeleSIP.AddCallPickUpGroupMember(comboPickupGroup.Text, sMemeberName);
            }

            if(!comboPickupGroup.Items.Contains(comboPickupGroup.Text))
                comboPickupGroup.Items.Add(comboPickupGroup.Text);

            MessageBox.Show("PickUp Group added/updated successfully.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        private void BtnRemoveCallPickupGroup_Click(object sender, EventArgs e)
        {
            if(comboPickupGroup.Text == "" || (!comboPickupGroup.Items.Contains(comboPickupGroup.Text)))
                return;
                        
            int nIndex = comboPickupGroup.FindStringExact(comboPickupGroup.Text);
            if(nIndex == -1) return;

            m_objVaxTeleSIP.RemoveCallPickUpGroup(comboPickupGroup.Text);
            comboPickupGroup.Items.RemoveAt(nIndex);

            ClearDlgContent();

            if (comboPickupGroup.Items.Count > 0)
                comboPickupGroup.SelectedIndex = 0;

            ShowPickupGroupDetails();

            MessageBox.Show("PickUp Group removed successfully.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        private void BtnClearCallPickupGroup_Click(object sender, EventArgs e)
        {
            comboPickupGroup.Text = "";
            comboAgentName.Text = "";

            ListBoxMember.Items.Clear();
        }

        private void BtnAddAgent_Click(object sender, EventArgs e)
        {
            if(ListBoxMember.FindStringExact(comboAgentName.Text) != -1)
            {
                MessageBox.Show("Login already exists.", "VaxTeleSIP", MessageBoxButtons.OK);
                return;
            }

            ListBoxMember.Items.Add(comboAgentName.Text);
        }

        private void BtnRemoveAgent_Click(object sender, EventArgs e)
        {
            ListBoxMember.Items.Remove(ListBoxMember.SelectedItem);
        }

        private void BtnClearAgent_Click(object sender, EventArgs e)
        {
            ListBoxMember.Items.Clear();
        }
        
        private void ClearDlgContent()
        { 
            comboPickupGroup.Text = "";
            ListBoxMember.Items.Clear();
        }

        private void AddUsers()
        {
            comboAgentName.Items.Clear();

            ArrayList objListUsers = new ArrayList();
            m_objVaxTeleSIP.GetUserAll(ref objListUsers);

            if(objListUsers.Count <= 0) return;

            for (int nIndex = 0; nIndex < objListUsers.Count; nIndex++)
                comboAgentName.Items.Add(objListUsers[nIndex]);

            if(comboAgentName.SelectedIndex == -1 && comboAgentName.Items.Count > 0)
                comboAgentName.SelectedIndex = 0;
        }

        private void AddPickupGroups()
        {
            comboPickupGroup.Items.Clear();

            List<string> objListPickUpName = new List<string>();
            m_objVaxTeleSIP.GetPickUpGroupNameAll(ref objListPickUpName);

            if(objListPickUpName.Count <= 0)
                return;

            foreach (string sPickUpGroupName in objListPickUpName)
            {
                comboPickupGroup.Items.Add(sPickUpGroupName);
            }

            if(comboPickupGroup.Items.Count > 0)
                comboPickupGroup.SelectedIndex = 0;
        }

        public void OnSetActive()
        {
            AddUsers();
            AddPickupGroups();

            ShowPickupGroupDetails();
        }

        private void OnCbnSelchangeComboCallPickup(object sender, EventArgs e)
        {
            ShowPickupGroupDetails();
        }

        private void ShowPickupGroupDetails()
        {
            String sPickUpGroupName = comboPickupGroup.Text;
            
            if (sPickUpGroupName.Length == 0)
                return;
            
            ListBoxMember.Items.Clear();

            List<string> objListCallPickupMember = new List<string>();
            if(!m_objVaxTeleSIP.GetPickUpMemberNameAll(sPickUpGroupName, ref objListCallPickupMember))
                return;

            foreach (string sMemberName in objListCallPickupMember)
                ListBoxMember.Items.Add(sMemberName);

        }

    }
}
