﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using ServerSIP;
using System.Collections;

namespace VaxTeleConfigSIP
{
    public partial class RoomForm : Form
    {

        private cVaxServerSIP m_objVaxTeleSIP;

        public RoomForm(ref cVaxServerSIP objVaxServer)
        {
            InitializeComponent();

            m_objVaxTeleSIP = objVaxServer;
        }

        private void OnAddConfRoom(object sender, EventArgs e)
        {
            if(!AddRoomChecks())
                return;

            if(ComboRoomName.FindStringExact(ComboRoomName.Text) != -1)
            {
                if(MessageBox.Show("Room already exists, do you want to continue?", "VaxTeleSIP", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == DialogResult.No)
                {
                    return;
                }
            }

            if(ComboRoomName.Items.Contains(ComboRoomName.Text))
                m_objVaxTeleSIP.RemoveRoom(ComboRoomName.Text.Trim());

            if(!m_objVaxTeleSIP.AddRoom(ComboRoomName.Text.Trim(), EditRoomNo.Text.Trim()))
            {
                MessageBox.Show("Room could not be added/updated or Please start the server first. or Can't assign same Room-No to two different rooms.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if(!ComboRoomName.Items.Contains(ComboRoomName.Text))
                ComboRoomName.Items.Add(ComboRoomName.Text);

            MessageBox.Show("Room added/updated successfully.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        private void OnRemoveConfRoom(object sender, EventArgs e)
        {
            m_objVaxTeleSIP.RemoveRoom(ComboRoomName.Text.Trim());

            int nIndex = ComboRoomName.FindStringExact(ComboRoomName.Text);
            if(nIndex == -1) return;

            m_objVaxTeleSIP.RemoveRoom(ComboRoomName.Text);
            ComboRoomName.Items.RemoveAt(nIndex);

            ClearDlgContent();

            if (ComboRoomName.Items.Count > 0)
                ComboRoomName.SelectedIndex = 0;

            ShowRoomDetails();

            MessageBox.Show("Room removed successfully!", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        private Boolean AddRoomChecks()
        {
            if(ComboRoomName.Text.Length == 0)
            {
                MessageBox.Show("Please enter Room Name.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            if(EditRoomNo.Text.Length == 0)
            {
                MessageBox.Show("Room No can't be empty.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            string sDigit = "0123456789";

            for (int nCount = 0; nCount < EditRoomNo.Text.Length; nCount++)
            {
                if(sDigit.IndexOf(EditRoomNo.Text[nCount]) == -1)
                {
                    MessageBox.Show("Room No should contain only numerals 0 through 9. For example: 12, 324323, 800122, 2109756 etc.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return false;
                }
            }
            
            return true;
        }

        private void ComboRoomName_SelectedIndexChanged(object sender, EventArgs e)
        {
            ShowRoomDetails();
        }

        private void ClearDlgContent()
        {
            EditRoomNo.Text = "";
        }

        private void ShowRoomDetails()
        {
            String sRoomName = ComboRoomName.Text;
            
            if (sRoomName.Length == 0)
                return;
            
            String sRoomNo = "";

            if(!m_objVaxTeleSIP.GetRoomDetails(sRoomName, ref sRoomNo))
                return;

            EditRoomNo.Text = sRoomNo;
        }

        private void AddRooms()
        {
            ComboRoomName.Items.Clear();

            ArrayList objListRooms = new ArrayList();
            m_objVaxTeleSIP.GetRoomAll(ref objListRooms);

            if(objListRooms.Count <= 0) return;

            for (int nCount = 0; nCount < objListRooms.Count; nCount++)
                ComboRoomName.Items.Add(objListRooms[nCount]);

            if(ComboRoomName.Items.Count > 0)
                ComboRoomName.SelectedIndex = 0;
        }

        public void OnSetActive()
        {
            AddRooms();
            ShowRoomDetails();
        }
    }
}
