// AddConferenceDlg.cpp : implementation file
//

#include "stdafx.h"
#include "VaxTeleServerApp.h"
#include "AddConferenceDlg.h"
#include "afxDialogEx.h"

#include "VaxServerSIP.h"	


// CAddConferenceDlg dialog

IMPLEMENT_DYNAMIC(CAddConferenceDlg, CPropertyPage)

CAddConferenceDlg::CAddConferenceDlg(CVaxServerSIP* pVaxTeleSIP) : CPropertyPage(CAddConferenceDlg::IDD)
{
	m_pVaxTeleSIP = pVaxTeleSIP;

	m_sRoomNo = _T("");
	m_sRoomName = _T("");
}

CAddConferenceDlg::~CAddConferenceDlg()
{
}

void CAddConferenceDlg::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_COMBO_CONFERENCE_ROOM_NAME, m_ctrlComboRoomName);
	DDX_Text(pDX, IDC_EDIT_CONFERENCE_ROOM_NO, m_sRoomNo);
	DDX_CBString(pDX, IDC_COMBO_CONFERENCE_ROOM_NAME, m_sRoomName);
}


BEGIN_MESSAGE_MAP(CAddConferenceDlg, CPropertyPage)
	ON_BN_CLICKED(IDC_BUTTON_CONFERENCE_ADD_ROOM, &CAddConferenceDlg::OnButtonAddRoom)
	ON_BN_CLICKED(IDC_BUTTON_CONFERENCE_REMOVE_ROOM, &CAddConferenceDlg::OnButtonRemoveRoom)
	ON_CBN_SELCHANGE(IDC_COMBO_CONFERENCE_ROOM_NAME, &CAddConferenceDlg::OnCbnSelchangeComboConferenceRoomName)
END_MESSAGE_MAP()


// CAddConferenceDlg message handlers

BOOL CAddConferenceDlg::OnSetActive()
{
	AddRooms();
	ShowRoomDetails();

	return CPropertyPage::OnSetActive();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CAddConferenceDlg::OnButtonAddRoom()
{
	UpdateData();

	if(!AddRoomChecks())
		return;

	if(m_ctrlComboRoomName.FindStringExact(0, m_sRoomName) != -1)
	{
		if(MessageBox("Room already exists, do you want to continue?", "VaxTeleSIP", MB_ICONEXCLAMATION|MB_YESNO) == IDNO)
			return;
	}

	m_pVaxTeleSIP->RemoveRoom(m_sRoomName);
	
	if(!m_pVaxTeleSIP->AddRoom(m_sRoomName, m_sRoomNo))
	{
		MessageBox("Room could not be added/updated or Please start the server first. or Can't assign same Room-No to two different rooms.", "VaxTeleSIP", MB_OK|MB_ICONINFORMATION);
		return;
	}
	
	if(m_ctrlComboRoomName.FindStringExact(0, m_sRoomName) == -1)
		m_ctrlComboRoomName.AddString(m_sRoomName);

	MessageBox("Room added/updated successfully.", "VaxTeleSIP", MB_OK|MB_ICONINFORMATION);
}

void CAddConferenceDlg::OnButtonRemoveRoom()
{
	UpdateData();
	
	if(m_sRoomName.IsEmpty())
	{
		MessageBox("Room Name can't be empty.", "VaxTeleSIP", MB_OK|MB_ICONINFORMATION);
		return;
	}

	int nIndex = m_ctrlComboRoomName.FindStringExact(0, m_sRoomName);
	if(nIndex == -1) return;

	m_pVaxTeleSIP->RemoveRoom(m_sRoomName);
	m_ctrlComboRoomName.DeleteString(nIndex);

	ClearDlgContent();
	
	if(m_ctrlComboRoomName.GetCount() > 0)
		m_ctrlComboRoomName.SetCurSel(0);
		
	ShowRoomDetails();

	MessageBox("Room removed successfully!", "VaxTeleSIP", MB_OK|MB_ICONINFORMATION);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CAddConferenceDlg::AddRooms()
{
	m_ctrlComboRoomName.ResetContent();

	CStringList sListRoomName;
	m_pVaxTeleSIP->GetRoomAll(&sListRoomName);
			
	while(!sListRoomName.IsEmpty())
	{
		m_ctrlComboRoomName.AddString(sListRoomName.RemoveHead());
	}

	if(m_ctrlComboRoomName.GetCount() > 0)
		m_ctrlComboRoomName.SetCurSel(0);
}

void CAddConferenceDlg::OnCbnSelchangeComboConferenceRoomName()
{
	ShowRoomDetails();
}

BOOL CAddConferenceDlg::AddRoomChecks()
{
	if(m_sRoomName.IsEmpty())
	{
		MessageBox("Room Name can't be empty.", "VaxTeleSIP", MB_OK|MB_ICONINFORMATION);
		return FALSE;
	}

	if(m_sRoomNo.IsEmpty())
	{
		MessageBox("Room No can't be empty.", "VaxTeleSIP", MB_OK|MB_ICONINFORMATION);
		return FALSE;
	}

	return TRUE;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CAddConferenceDlg::ShowRoomDetails()
{
	int nSelectedIndex = m_ctrlComboRoomName.GetCurSel();
	if(nSelectedIndex == -1) return;

	CString sRoomName;
	
	m_ctrlComboRoomName.GetLBText(nSelectedIndex, sRoomName);
	if(sRoomName.IsEmpty()) return;
			
	CString sRoomNo;
	
	if(!m_pVaxTeleSIP->GetRoomDetails(sRoomName, sRoomNo))
		return;
	
	m_sRoomName = sRoomName;
	m_sRoomNo = sRoomNo;

	UpdateData(FALSE);
}

void CAddConferenceDlg::ClearDlgContent()
{
	m_sRoomName = "";
	m_sRoomNo = "";

	UpdateData(FALSE);
}

