﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

namespace ServerSIP
{
    public class cVaxServerExCOM : cVaxServerCOM
    {
        int m_nDialToneId = -1;
        int m_nRegisterUserCount = 0;

        Boolean m_bInitialized = false;

        cVaxCallSessionRoute m_objRouteCallSIP = null;
        cConfRoom m_objConfRoom = null;
        cVaxChat m_objChat = null;

        #region Methods

        public cVaxServerExCOM()
        {
            m_objConfRoom = new cConfRoom(this);
            m_objRouteCallSIP = new cVaxCallSessionRoute(this);
            m_objChat = new cVaxChat(this);
        }

        ~cVaxServerExCOM()
        {

        }
        
	    public new bool Initialize(string  sDomainRealm, string  sSIPListenIP, int nSIPListenPort, string  sRTPListenIP, int nRTPListenPort)
        {
            if (!base.Initialize(sDomainRealm, sSIPListenIP, nSIPListenPort, sRTPListenIP, nRTPListenPort))
                return false;

            m_bInitialized = true;
            OnInitialize();
            
            return true;
        }
	    
        public new void UnInitialize()
        {
            m_bInitialized = false;
            
            m_nRegisterUserCount = 0;

            DeleteUserDataAll();
            DeleteCallSessionDataAll();

            m_objConfRoom.DeleteRoomAll();
            base.UnInitialize();
        }

        public bool IsInitialized()
        {
            return m_bInitialized;
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

        public bool AddUser(string sUserName, string sPassword, string sCodecList, bool bBlocked, string sUserPrefix, string sDialPrefix, int nToPeerType, string sToPeerName)
        {
	        if(!base.AddUser(sUserName, sPassword, sCodecList))	
	        {
		        return false;
	        }

            cVaxUser objUser = new cVaxUser(this);
	        base.SetUserData(sUserName, objUser);

            objUser.SetData(sPassword, sCodecList, bBlocked, sUserPrefix, sDialPrefix, nToPeerType, sToPeerName);

	        return true;
        }

	    public new void RemoveUser(string  sUserName)
        {
            if (!IsUserExist(sUserName))
                return;

            base.FreeUserData(sUserName);
            base.RemoveUser(sUserName);
        }

        private bool IsUserExist(string sUserName)
        {
            if (!base.TrvsUserStart())
                return false;

            while (true)
            {
                string sUser = base.TrvsUserNext();
                if (sUser.Length == 0) break;

                if (sUser == sUserName)
                    return true;
            }

            return false;
        }

        public void GetUserAll(ref ArrayList objList)
        {
            base.TrvsUserStart();

            while (true)
            {
                string sUserName = base.TrvsUserNext();
		        if(sUserName.Length == 0) break;

                objList.Add(sUserName);
            }
        }

        public bool GetUserDetails(string sUserName, ref string sPassword, ref string sCodecList, ref bool bBlocked, ref string sUserPrefix, ref string sDialPrefix, ref int nToPeerType, ref string sToPeerName)
        {
            cVaxUser objUser = (cVaxUser)base.GetUserData(sUserName);
            if (objUser == null) return false;

            objUser.GetData(ref sPassword, ref sCodecList, ref bBlocked, ref sUserPrefix, ref sDialPrefix, ref nToPeerType, ref sToPeerName);
            return true;
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

        public bool AddLine(string sLineName, string sDisplayName, string sUserName, string sAuthUser, string sAuthPwd, string sDomainRealm, string sProxySIP, string sCodecList, bool bRegister, int nToPeerType, string sToPeerName, bool bBlock, string sRoomName)
        {
	        if(!base.AddLine(sLineName, sDisplayName, sUserName, sAuthUser, sAuthPwd, sDomainRealm, sProxySIP, "", sCodecList))
	        {
		        return false;
	        }

            cVaxLine objLine = new cVaxLine(this);

            base.SetLineData(sLineName, objLine);
            objLine.SetData(sDisplayName, sUserName, sAuthUser, sAuthPwd, sDomainRealm, sProxySIP, sCodecList, bRegister, nToPeerType, sToPeerName, bBlock, sRoomName);

            if (bRegister)
                base.RegisterLine(sLineName, 1800);

             return true;
        }
	    
        public new void RemoveLine(string  sLineName)
        {
            if (!IsLineExist(sLineName))
                return;

            base.FreeLineData(sLineName);
            base.RemoveLine(sLineName);
        }

        private bool IsLineExist(string sLineName)
        {
            if (!base.TrvsLineStart())
                return false;

            while (true)
            {
                string sLine = base.TrvsLineNext();
                if (sLine.Length == 0) break;

                if (sLine == sLineName)
                    return true;
            }

            return false;
        }
                
        public void GetLineAll(ref ArrayList objList)
        {
            base.TrvsLineStart();

            while (true)
            {
                string sLineName = base.TrvsLineNext();
                if (sLineName.Length == 0) break;

                objList.Add(sLineName);
            }
        }

        public bool GetLineDetails(string sLineName, ref string sDisplayName, ref string sUserName, ref string sAuthUser, ref string sAuthPwd, ref string sDomainRealm, ref string sProxySIP, ref string sCodecList, ref bool bRegister, ref int nToPeerType, ref string sToPeerName, ref bool bRejected, ref string sRoomName)
        {
            cVaxLine objLine = (cVaxLine)base.GetLineData(sLineName);
            if (objLine == null) return false;

            objLine.GetData(ref sDisplayName, ref sUserName, ref sAuthUser, ref sAuthPwd, ref sDomainRealm, ref sProxySIP, ref sCodecList, ref bRegister, ref nToPeerType, ref sToPeerName, ref bRejected, ref sRoomName);
            return true;
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        
        public bool AddDirectProxySIP(string sProxyName, string sAuthLogin, string sAuthPwd, string sDomainRealm, string sProxyIP, int nProxyPort, string sCodecList, int nToPeerType, string sToPeerName, bool bRejected, string sRoomName)
        {
            if (!base.AddDirectProxySIP(sProxyName, sAuthLogin, sAuthPwd, sDomainRealm, sProxyIP, nProxyPort, sCodecList))
	        {
		        return false;
	        }

            cVaxDirectProxy objDirectProxy = new cVaxDirectProxy(this);
            base.SetDirectProxyData(sProxyName, objDirectProxy);

            objDirectProxy.SetData(sAuthLogin, sAuthPwd, sDomainRealm, sProxyIP, nProxyPort, sCodecList, nToPeerType, sToPeerName, bRejected, sRoomName);
            return true;
        }

	    public new void RemoveDirectProxySIP(string  sProxyName)
        {
            if(!IsDirectProxyExist(sProxyName))
                return;

            base.FreeDirectProxyData(sProxyName);
            base.RemoveDirectProxySIP(sProxyName);
        }

        private bool IsDirectProxyExist(string sProxyName)
        {
            if (!base.TrvsDirectProxyStart())
                return false;

            while (true)
            {
                string sProxy = base.TrvsDirectProxyNext();
                if (sProxy.Length == 0) break;

                if (sProxy == sProxyName)
                    return true;
            }

            return false;
        }

        public void GetDirectProxyAll(ref ArrayList objList)
        {
            base.TrvsDirectProxyStart();

            while (true)
            {
                string sProxyName = base.TrvsDirectProxyNext();
                if (sProxyName.Length == 0) break;

                objList.Add(sProxyName);
            }
        }

        public bool GetDirectProxyDetails(string sProxyName, ref string sAuthLogin, ref string sAuthPwd, ref string sDomainRealm, ref string sProxyIP, ref int nProxyPort, ref string sCodecList, ref int nToPeerType, ref string sToPeerName, ref bool bBlocked, ref string sRoomName)
        {
            cVaxDirectProxy objDirectProxy = (cVaxDirectProxy)base.GetDirectProxyData(sProxyName);
            if (objDirectProxy == null) return false;

            objDirectProxy.GetData(ref sAuthLogin, ref sAuthPwd, ref sDomainRealm, ref sProxyIP, ref nProxyPort, ref sCodecList, ref nToPeerType, ref sToPeerName, ref bBlocked, ref sRoomName);
            return true;
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                
        public new int GetVaxErrorCode()
        {
        	return base.GetVaxErrorCode();
        }

        public bool DialToneToCallSession(bool bEnable, int nSessionId)
        {
            if (m_nDialToneId == -1)
                return true;

            return base.DialToneToCallSession(bEnable, nSessionId, m_nDialToneId);
        }

        public new int GetCallCount()
        {
            return base.GetCallCount();
        }

        public int GetRegisteredUserCount()
        {
	        return m_nRegisterUserCount;
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

        public bool AddRoom(string sRoomName, string sRoomNo)
        {
            return m_objConfRoom.AddRoom(sRoomName, sRoomNo);
        }

        public void RemoveRoom(string sRoomName)
        {
            m_objConfRoom.RemoveRoom(sRoomName);
        }

        public bool GetRoomDetails(string sRoomName, ref string sRoomNo)
        {
            return m_objConfRoom.GetRoomDetails(sRoomName, ref sRoomNo);
        }

        public bool IsRoomNoExists(string sRoomNo)
        {
            return m_objConfRoom.IsRoomNoExists(sRoomNo);
        }

        public bool GetRoomAll(ref ArrayList aRoomList)
        {
            return m_objConfRoom.GetRoomAll(ref aRoomList);
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

        public bool LoadMusicDialTone(string sFileName)
        {
            UnLoadMusicDialTone();

            m_nDialToneId = base.LoadWaveFile(sFileName);
            return (m_nDialToneId != -1);
        }

        public void UnLoadMusicDialTone()
        {
            if (m_nDialToneId != -1)
            {
                base.UnLoadWaveFile(m_nDialToneId);
                m_nDialToneId = -1;
            }
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

        public bool AddUserRoute(string sUserName, string sUserPrefix, string sDialPrefix, int nToPeerType, string sToPeerName)
        {
            return m_objRouteCallSIP.AddUserRoute(sUserName, sUserPrefix, sDialPrefix, nToPeerType, sToPeerName);
        }

        public void RemoveUserRoute(string sUserName)
        {
            m_objRouteCallSIP.RemoveUserRoute(sUserName);
        }

        public bool AddLineRoute(string sLineName, int nToPeerType, string sToPeerName, bool bRejected,string sRoomName)
        {
            return m_objRouteCallSIP.AddLineRoute(sLineName, nToPeerType, sToPeerName, bRejected, sRoomName);
        }

        public void RemoveLineRoute(string sLineName)
        {
            m_objRouteCallSIP.RemoveLineRoute(sLineName);
        }

        public bool AddDirectProxyRoute(string sProxyName, int nToPeerType, string sToPeerName, bool bRejected, string sRoomName)
        {
            return m_objRouteCallSIP.AddDirectProxyRoute(sProxyName, nToPeerType, sToPeerName, bRejected, sRoomName);
        }

        public void RemoveDirectProxyRoute(string sProxyName)
        {
            m_objRouteCallSIP.RemoveDirectProxyRoute(sProxyName);
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                
        public void DeleteCallSessionDataAll()
        {
            base.TrvsCallSessionStart();
		    
	        for(int nCount = 0; nCount < base.GetCallSessionCount(); nCount++)
	        {
		        int nSessionId = base.TrvsCallSessionNext();
		        base.FreeCallSessionData(nSessionId);
	        }
        }

        public void DeleteUserDataAll()
        {
            base.TrvsUserStart();
		    
	        for(int nCount = 0; nCount < base.GetUserCount(); nCount++)
	        {
		        string sUserName = base.TrvsUserNext();
		        base.FreeUserData(sUserName);
	        }
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


        #endregion

        #region Events

        public override void OnCallSessionCreated(int nSessionId, int nReasonCode)
        {
            cVaxCallSession objCallSession = new cVaxCallSession(this);
            base.SetCallSessionData(nSessionId, objCallSession);
        }

        public override void OnCallSessionClosed(int nSessionId, int nChannelId, int nReasonCode)
        {
            base.FreeCallSessionData(nSessionId);
        }
        
        public override void OnIncomingCall(int nSessionId, string sCallerName, string sCallerId, string sDialNo, int nFromPeerType, string sFromPeerName, string sUserAgentName,  string sFromIP, int nFromPort)
        {
            string sToPeerName = "", sDialToNo = "", sRoomName = "";
            int nToPeerType = -1; bool bRejected = false;

            if (nFromPeerType == cVaxServerCOM.VAX_PEER_TYPE_USER)
            {
                if (m_objConfRoom.IsRoomNoExists(sDialNo))
                    sRoomName = m_objConfRoom.GetRoomName(sDialNo);
                else
                    m_objRouteCallSIP.OnCallFromUser(sFromPeerName, sDialNo, ref sDialToNo, ref nToPeerType, ref sToPeerName);
            }
            else if (nFromPeerType == cVaxServerCOM.VAX_PEER_TYPE_LINE)
            {
                m_objRouteCallSIP.OnCallFromLine(sFromPeerName, ref nToPeerType, ref sToPeerName, ref bRejected, ref sRoomName);
                sDialToNo = sToPeerName;
            }
            else if (nFromPeerType == cVaxServerCOM.VAX_PEER_TYPE_DIRECT_PROXY)
            {
                m_objRouteCallSIP.OnCallFromDirectProxySIP(sFromPeerName, ref nToPeerType, ref sToPeerName, ref bRejected, ref sRoomName);
                sDialToNo = sToPeerName;
            }

            //////////////////////////////////////////////////////////////////////

            if (bRejected == true)
            {
                base.RejectCallSession(nSessionId, 404, "Not Found");
                return;
            }

            DialToneToCallSession(true, nSessionId);

            cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nSessionId);
            objCallSession.OnIncomingCall(nSessionId, sCallerName, sCallerId, sDialNo, sRoomName, nFromPeerType, sFromPeerName, nToPeerType, sToPeerName, sUserAgentName, sFromIP, nFromPort);
        }
   
        public override void OnCallSessionConnected(int nSessionId)
        {
            cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nSessionId);
            objCallSession.OnCallSessionConnected(nSessionId);
        }

        public override void OnCallSessionConnecting(int nSessionId, int nChannelId, int nStatusCode, string sReasonPhrase)
        {
            cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nSessionId);
            objCallSession.OnCallSessionConnecting(nSessionId, nChannelId, nStatusCode, sReasonPhrase);
        }

        public override void OnCallSessionFailed(int nSessionId, int nChannelId, int nStatusCode, string sReasonPhrase)
        {
            cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nSessionId);
            objCallSession.OnCallSessionFailed(nSessionId, nChannelId, nStatusCode, sReasonPhrase);
        }

        public override void OnCallSessionOnHold(int nSessionId, int nChannelId)
        {
            cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nSessionId);
            objCallSession.OnCallSessionOnHold(nSessionId, nChannelId);
        }

        public override void OnCallSessionOffHold(int nSessionId, int nChannelId)
        {
            cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nSessionId);
            objCallSession.OnCallSessionOffHold(nSessionId, nChannelId);
        }

        public override void OnCallSessionDialToneStart(int nSessionId, int nChannelId, int nWaveId)
        {
        }

        public override void OnCallSessionDialToneStop(int nSessionId, int nChannelId, int nWaveId)
        {
        }

        public override void OnCallSessionPlayWaveDone(int nSessionId, int nChannelId, int nWaveId)
        {
        }

        ////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////

        public override void OnCallSessionTransferBlind(int nTransfererSessionId, int nTransfererChannelId, string sTransferer, string sTransferee, string sTransferTo)
        {
            cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nTransfererSessionId);
            objCallSession.OnCallSessionTransferBlind(nTransfererSessionId, nTransfererChannelId, sTransferer, sTransferee, sTransferTo);
        }
        public override void OnCallSessionTransferConsult(int nTransfererSessionId, int nTransfererChannelId, int nTransferToSessionId, int nTransferToChannelId, string sTransferer, string sTransferee, string sTransferTo)
        {
            cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nTransfererSessionId);
            objCallSession.OnCallSessionTransferConsult(nTransfererSessionId, nTransfererChannelId, nTransferToSessionId, nTransferToChannelId, sTransferer, sTransferee, sTransferTo);
        }

        ////////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////////

        public override void OnDetectedDigitDTMF(int nSessionId, int nChannelId, string sDigitDTMF)
        {
            cVaxCallSession objCallSession = (cVaxCallSession)base.GetCallSessionData(nSessionId);
            objCallSession.OnDetectedDigitDTMF(nSessionId, nChannelId, sDigitDTMF);
        }
        
        //////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////

        public override void OnLineRegisterTrying(string sLineName)
        {
            cVaxLine objLine = (cVaxLine)base.GetLineData(sLineName);
            if (objLine == null) return;

            objLine.OnLineRegisterTrying(sLineName);
        }
        public override void OnLineRegisterFailed(string sLineName, int nStatusCode, string sReasonPhrase)
        {
            cVaxLine objLine = (cVaxLine)base.GetLineData(sLineName);
            if (objLine == null) return;

            objLine.OnLineRegisterFailed(sLineName, nStatusCode, sReasonPhrase);
        }
        public override void OnLineRegisterSuccess(string sLineName)
       {
            cVaxLine objLine = (cVaxLine)base.GetLineData(sLineName);
            if (objLine == null) return;

            objLine.OnLineRegisterSuccess(sLineName);
        }
        
        //////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////

        public override void OnRegisterUser(string sUserLogin, string sDomain, string sUserAgentName, string sFromIP, int nFromPort, int nRegId)
        {
            cVaxUser objUser = (cVaxUser)base.GetUserData(sUserLogin);

            if (objUser == null)
            {
                RejectRegister(nRegId, 404, "Not Found");
                return;
            }
                                
            objUser.OnRegisterUser(sUserLogin, sDomain, sUserAgentName, sFromIP, nFromPort, nRegId);
        }

        public override void OnRegisterUserSuccess(string sUserLogin, string sFromIP, int nFromPort, int nRegId)
        {
            cVaxUser objUser = (cVaxUser)base.GetUserData(sUserLogin);

            if (!objUser.IsRegistered())
                m_nRegisterUserCount++;

            objUser.OnRegisterUserSuccess(sUserLogin, sFromIP, nFromPort, nRegId);
        }

        public override void OnRegisterUserFailed(string sUserLogin, string sFromIP, int nFromPort, int nRegId)
        {
            cVaxUser objUser = (cVaxUser)base.GetUserData(sUserLogin);
            objUser.OnRegisterUserFailed(sUserLogin, sFromIP, nFromPort, nRegId);
        }

        public override void OnUnRegisterUser(string sUserLogin)
        {
            cVaxUser objUser = (cVaxUser)base.GetUserData(sUserLogin);

            if (objUser.IsRegistered())
                m_nRegisterUserCount--;

            objUser.OnUnRegisterUser(sUserLogin);
        }

        //////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////

        public override void OnChatMessageText(int nChatMsgId, string sMsgFrom, string sMsgTo, string sMsgText, int nFromPeerType, string sFromPeerName, string sFromIP, int nFromPort)
        {
            m_objChat.OnChatMessageText(nChatMsgId, sMsgFrom, sMsgTo, sMsgText, nFromPeerType, sFromPeerName, sFromIP, nFromPort);
        }
        public override void OnChatMessageTyping(int nChatMsgId, string sMsgFrom, string sMsgTo, bool bIsTypingStart, int nFromPeerType, string sFromPeerName, string sFromIP, int nFromPort)
        {
            m_objChat.OnChatMessageTyping(nChatMsgId, sMsgFrom, sMsgTo, bIsTypingStart, nFromPeerType, sFromPeerName, sFromIP, nFromPort);
        }
        public override void OnChatStatusSubscribe(int nSubscrbId, string sMsgFrom, string sMsgTo, int nFromPeerType, string sFromPeerName, string sFromIP, int nFromPort)
        {
            m_objChat.OnChatStatusSubscribe(nSubscrbId, sMsgFrom, sMsgTo, nFromPeerType, sFromPeerName, sFromIP, nFromPort);
        }

        public override void OnChatMessageSuccess(int nChatMsgId)
        {
            m_objChat.OnChatMessageSuccess(nChatMsgId);
        }
        public override void OnChatMessageFailed(int nChatMsgId, int nStatusCode, string sReasonPhrase)
        {
            m_objChat.OnChatMessageFailed(nChatMsgId, nStatusCode, sReasonPhrase);
        }
        public override void OnChatMessageTimeout(int nChatMsgId)
        {
            m_objChat.OnChatMessageTimeout(nChatMsgId);
        }

        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public override void OnBusyLampSubscribe(int nSubScrbId, string sUserName, string sToUserName, string sFromIP, int nFromPort)
        {
            base.BusyLampSubscribeAccept(nSubScrbId, true, 30);
        }

        public override void OnBusyLampSendStatus(string sFromUserName, string sToUserName, int nStateId)
        {
            base.BusyLampSendStatus(sFromUserName, sToUserName, nStateId);
        }

        public override void OnBusyLampUnSubscribe(int nSubScrbId, string sUserName, string sToUserName, string sFromIP, int nFromPort)
        {
        }

        public override void OnBusyLampSubscribeSuccess(int nSubScrbId, string sUserName)
        {
        }

        public override void OnBusyLampSubscribeFailed(int nSubScrbId, string sUserName)
        {
        }


        //////////////////////////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////////////////////////

        public void GenStatusLog(string sStatus)
        {
	        OnStatusLog(sStatus);
        }

        public override void OnVaxErrorLog(string sFuncName, int nErrorCode, string sErrorMsg)
        {
            string sMsg = "[Method: " + sFuncName + "] [Error Msg: " + sErrorMsg + "] [Code: " + nErrorCode.ToString() + "]";
            OnErrorLog(sMsg);
        }

        public virtual void OnStatusLog(string sMsg)
        {

        }

        public virtual void OnErrorLog(string sMsg)
        {

        }

        public virtual void OnInitialize()
        {

        }
                
        #endregion

    }
}
