﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using ServerSIP;

namespace VaxTeleConfigSIP
{
    public partial class DirectProxyForm : Form
    {
        private cVaxServerSIP m_objVaxTeleSIP;

        public DirectProxyForm(ref cVaxServerSIP objVaxServer)
        {
            InitializeComponent();

            m_objVaxTeleSIP = objVaxServer;
        }

        private void DirectProxyForm_Load(object sender, EventArgs e)
        {
            LoadCodecs();
        }

        private void ClearDlgContent()
        {
            int nProxyPort = 5060;

            ComboProxy.Text = "";

            EditAuthUser.Text = "";
            EditPassword.Text = "";
            EditDomainRealm.Text = "";
            EditProxySIP.Text = "";
            EditProxyPort.Text = nProxyPort.ToString();

            if (ComboConnectToUser.Items.Count > 0)
                ComboConnectToUser.SelectedIndex = 0;

            if (ComboPutInConf.Items.Count > 0)
                ComboPutInConf.SelectedIndex = 0;

            RadioRejectCall.Checked = true;

            ListBoxEnableCodec.Items.Clear();
            ListBoxDisableCodec.Items.Clear();

            ListBoxEnableCodec.Items.Add(cCodec.G711U_TEXT);
            ListBoxEnableCodec.Items.Add(cCodec.G711A_TEXT);
            ListBoxEnableCodec.Items.Add(cCodec.ILBC_TEXT);
            ListBoxEnableCodec.Items.Add(cCodec.G610_TEXT);
            ListBoxEnableCodec.Items.Add(cCodec.G729_TEXT);
        }

        private void BtnAddProxy_Click(object sender, EventArgs e)
        {
            if (AddProxyChecks() == false)
                return;

            string sProxyName = ComboProxy.Text.Trim().ToString();

            if (ComboProxy.FindStringExact(ComboProxy.Text) != -1)
            {
                if (MessageBox.Show("Direct proxy already exists, do you want to continue?", "VaxTeleSIP", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) == DialogResult.No)
                {
                    return;
                }
            }

            cCodec objCodec = new cCodec();
            string sCodec = "";

            for (int nCount = 0; nCount < ListBoxEnableCodec.Items.Count; nCount++)
                sCodec += objCodec.GetCodecNo((string)ListBoxEnableCodec.Items[nCount]);

            ///////////////////////////////////////////////////////////////////////////////////////////////////////

            int nToPeerType = -1;
            string sToPeerName = "", sRoomName = "";
            bool bRejected = false;

            if (RadioRejectCall.Checked)
                bRejected = true;

            else if (RadioConnectToUser.Checked)
            {
                nToPeerType = cVaxServerCOM.VAX_PEER_TYPE_USER;
                sToPeerName = ComboConnectToUser.Text.Trim().ToString();
            }
            else if (RadioPutInConf.Checked)
            {
                sRoomName = ComboPutInConf.Text.Trim().ToString();
            }

            ///////////////////////////////////////////////////////////////////////////////////////////////////////

            if (ComboProxy.Items.Contains(ComboProxy.Text))
                m_objVaxTeleSIP.RemoveDirectProxySIP(sProxyName);

            if (m_objVaxTeleSIP.AddDirectProxySIP(sProxyName, EditAuthUser.Text, EditPassword.Text, EditDomainRealm.Text, EditProxySIP.Text, Convert.ToInt32(EditProxyPort.Text), sCodec, nToPeerType, sToPeerName, bRejected, sRoomName) == false)
            {
                MessageBox.Show("Direct-Proxy could not be created or Please start the server first.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            if (!ComboProxy.Items.Contains(ComboProxy.Text))
                ComboProxy.SelectedIndex = ComboProxy.Items.Add(sProxyName);

            MessageBox.Show("Direct-Proxy added/updated successfully.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        private void BtnRemoveProxy_Click(object sender, EventArgs e)
        {
            if (ComboProxy.Text.Length == 0)
            {
                MessageBox.Show("Please enter proxy name.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

           int nIndex = ComboProxy.FindStringExact(ComboProxy.Text);
            if (nIndex == -1) return;

            m_objVaxTeleSIP.RemoveDirectProxySIP(ComboProxy.Text);
            ComboProxy.Items.RemoveAt(nIndex);

            ClearDlgContent();

            if (ComboProxy.Items.Count > 0)
                ComboProxy.SelectedIndex = 0;

            ShowProxyDetails();

            MessageBox.Show("Direct-Proxy removed successfully!", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        private void BtnClearProxyDlg_Click(object sender, EventArgs e)
        {
            ClearDlgContent();
        }

        private void BtnDisableCodec_Click(object sender, EventArgs e)
        {
            if (ListBoxEnableCodec.SelectedItems.Count == 0) return;

            ListBoxDisableCodec.Items.Add(ListBoxEnableCodec.SelectedItem.ToString());
            ListBoxEnableCodec.Items.Remove(ListBoxEnableCodec.SelectedItem);
        }

        private void BtnEnableCodec_Click(object sender, EventArgs e)
        {
            if (ListBoxDisableCodec.SelectedItems.Count == 0) return;

            ListBoxEnableCodec.Items.Add(ListBoxDisableCodec.SelectedItem.ToString());
            ListBoxDisableCodec.Items.Remove(ListBoxDisableCodec.SelectedItem);
        }

        private void AddUsers()
        {
            ComboConnectToUser.Items.Clear();

            ArrayList objListUsers = new ArrayList();
            m_objVaxTeleSIP.GetUserAll(ref objListUsers);

            if (objListUsers.Count <= 0) return;

            for (int nIndex = 0; nIndex < objListUsers.Count; nIndex++)
                ComboConnectToUser.Items.Add(objListUsers[nIndex]);

            if (ComboConnectToUser.Items.Count > 0)
                ComboConnectToUser.SelectedIndex = 0;
        }

        private void AddRooms()
        {
            ComboPutInConf.Items.Clear();

            ArrayList objListConfs = new ArrayList();
            m_objVaxTeleSIP.GetRoomAll(ref objListConfs);

            if (objListConfs.Count <= 0) return;

            for (int nIndex = 0; nIndex < objListConfs.Count; nIndex++)
                ComboPutInConf.Items.Add(objListConfs[nIndex]);

            if (ComboPutInConf.Items.Count > 0)
                ComboPutInConf.SelectedIndex = 0;
        }

        private void AddDirectProxies()
        {
            RadioRejectCall.Checked = true;

            ComboProxy.Items.Clear();

            ArrayList objProxyList = new ArrayList();
            m_objVaxTeleSIP.GetDirectProxyAll(ref objProxyList);

            if (objProxyList.Count <= 0) return;
            
            for (int nIndex = 0; nIndex < objProxyList.Count; nIndex++)
                ComboProxy.Items.Add(objProxyList[nIndex]);

            ComboProxy.SelectedIndex = 0;
        }

        private void LoadCodecs()
        {
            cCodec objCodec = new cCodec();

            for (int nCount = 0; nCount < cCodec.TOTAL_SIZE_CODEC; nCount++)
                ListBoxEnableCodec.Items.Add(objCodec.GetCodecName(Convert.ToString(nCount)));
        }

        private void ComboProxy_SelectedIndexChanged(object sender, EventArgs e)
        {
            ShowProxyDetails();
        }

        private Boolean AddProxyChecks()
        {

            if (ComboProxy.Text.Equals(""))
            {
                MessageBox.Show("Please enter a proxy name", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            if (ListBoxEnableCodec.Items.Count == 0)
            {
                MessageBox.Show("Please select a codec.", "VaxTeleSIP", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }

            return true;
        }

        public void OnSetActive()
        {
            AddUsers();
            AddRooms();
            AddDirectProxies();

            ShowProxyDetails();
        }

        private void UpdateDlgContent(string sAuthUser, string sAuthPwd, string sRealm, string sProxySIP, string sCodecList, long lProxyPort, int nToPeerType, string sToPeerName, bool bBlock, string sRoomName)
        {

            EditAuthUser.Text = sAuthUser;
            EditPassword.Text = sAuthPwd;
            EditDomainRealm.Text = sRealm;
            EditProxySIP.Text = sProxySIP;
            EditProxyPort.Text = lProxyPort.ToString();

            UpdateIncomingCallSection(nToPeerType, sToPeerName, bBlock, sRoomName);

            ListBoxEnableCodec.Items.Clear();
            ListBoxDisableCodec.Items.Clear();

            cCodec objCodec = new cCodec();

            for (int nCount = 0; nCount < cCodec.TOTAL_SIZE_CODEC; nCount++)
                ListBoxDisableCodec.Items.Add(objCodec.GetCodecName(Convert.ToString(nCount)));

            string sCodecName;

            for (int nCount = 0; nCount < sCodecList.Length; nCount++)
            {
                sCodecName = objCodec.GetCodecName(sCodecList[nCount].ToString());
                ListBoxEnableCodec.Items.Add(sCodecName);
                ListBoxDisableCodec.Items.Remove(sCodecName);
            }
        }

        private void UpdateIncomingCallSection(int nToPeerType, string sToPeerName, bool bRejected, string sRoomName)
        {
            if (bRejected)
            {
                RadioRejectCall.Checked = true;
            }
            else if (nToPeerType == cVaxServerCOM.VAX_PEER_TYPE_USER)
            {
                int nIndex = ComboConnectToUser.FindStringExact(sToPeerName);

                if (nIndex == -1)
                {
                    RadioRejectCall.Checked = true;
                    return;
                }

                if (ComboConnectToUser.Items.Count > 0)
                    ComboConnectToUser.SelectedIndex = nIndex;

                RadioConnectToUser.Checked = true;
            }
            else if (sRoomName.Length != 0)
            {
                int nIndex = ComboPutInConf.FindStringExact(sRoomName);

                if (nIndex == -1)
                {
                    RadioRejectCall.Checked = true;
                    return;
                }

                if (ComboPutInConf.Items.Count > 0)
                    ComboPutInConf.SelectedIndex = nIndex;

                RadioPutInConf.Checked = true;
            }
        }

        private void ShowProxyDetails()
        {
            String sProxyName = ComboProxy.Text;

            if (sProxyName.Length == 0)
                return;
            
            string sAuthUser = "", sAuthPwd = "";
            string sDomainRealm = "", sProxySIP = "";
            string sCodecList = "", sRoomName = "", sToPeerName = "";

            bool bRejected = false;
            int nProxyPort = -1; int nToPeerType = -1;

            m_objVaxTeleSIP.GetDirectProxyDetails(sProxyName, ref sAuthUser, ref sAuthPwd, ref sDomainRealm, ref sProxySIP, ref nProxyPort, ref sCodecList, ref nToPeerType, ref sToPeerName, ref bRejected, ref sRoomName);
            UpdateDlgContent(sAuthUser, sAuthPwd, sDomainRealm, sProxySIP, sCodecList, nProxyPort, nToPeerType, sToPeerName, bRejected, sRoomName);
        }

    }
}
